package net.hostsharing.hsadminng.hs.migration;

import liquibase.Liquibase;
import net.hostsharing.hsadminng.context.Context;
import net.hostsharing.hsadminng.rbac.test.JpaAttempt;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Tag;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.autoconfigure.orm.jpa.DataJpaTest;
import org.springframework.context.annotation.Import;
import org.springframework.test.annotation.DirtiesContext;
import org.springframework.test.context.ActiveProfiles;
import org.springframework.test.context.TestPropertySource;
import org.springframework.test.context.jdbc.Sql;

import javax.sql.DataSource;

import static org.assertj.core.api.Assertions.assertThat;
import static org.springframework.test.context.jdbc.Sql.ExecutionPhase.BEFORE_TEST_CLASS;

/** Tests, if the Liquibase scripts can be applied to a database ionitialized with schemas
    and test-data from a previous version.

    <p>The test needs a dump, ideally from the version of the lastest prod-release:</p>

    <ol>
        <li>1. clean the database:<br/>
            <code>pg-sql-reset</code>

        <li>populate the database:</br>
            <code>./gradlew bootRun --args='--spring.profiles.active=only-office'</code>

        <li>create the reference-schema SQL-file with some initializations:</li>
            <pre><code>cat >src/test/resources/db/prod-only-office-schema-with-test-data.sql <<EOF
            CREATE ROLE postgres;

            CREATE ROLE admin;
            GRANT ALL PRIVILEGES ON ALL TABLES IN SCHEMA public TO admin;
            CREATE ROLE restricted;
            GRANT ALL PRIVILEGES ON ALL TABLES IN SCHEMA public TO restricted;

            EOF
         </code></pre>

        <li>add the dump to that reference-schema SQL-file:</p>
            <pre><code>docker exec -i hsadmin-ng-postgres /usr/bin/pg_dump \
                --column-inserts --disable-dollar-quoting -U postgres postgres \
               >>src/test/resources/db/prod-only-office-schema-with-test-data.sql
            </code></pre>
    </ol>

    <p>The generated dump has to be committed to git and will be used in future test-runs
    until it gets replaced at the next release.</p>
*/
@Tag("importOfficeData")
@DataJpaTest(properties = {
        "spring.liquibase.enabled=false" // @Sql should go first, Liquibase will be initialized programmtically
})
@ActiveProfiles("only-office")
@TestPropertySource(properties = "spring.liquibase.contexts=only-office")
@DirtiesContext
@Import({ Context.class, JpaAttempt.class, LiquibaseConfig.class})
@Sql(value = "/db/prod-only-office-schema-with-test-data.sql", executionPhase = BEFORE_TEST_CLASS)
public class LiquibaseCompatibilityIntegrationTest extends CsvDataImport {

    @Autowired
    private DataSource dataSource;

    @Autowired
    private Liquibase liquibase;

    @BeforeEach
    public void setup() throws Exception {

        final var schemas = em.createNativeQuery("SELECT * FROM pg_catalog.pg_tables WHERE schemaname='public'").getResultList();
        assertThat(schemas).hasSize(2);

        final var liquibaseScripts = em.createNativeQuery("SELECT * FROM public.databasechangelog").getResultList();
        assertThat(liquibaseScripts).hasSize(286);

        // Step 2: Run Liquibase migrations
        liquibase.update(new liquibase.Contexts(), new liquibase.LabelExpression());
    }

    @Test
    void test() {
        // FIXME: check if changes got applied
    }
}
